@@ -1,30 +1,46 @@
 package Proc::Exists;
 
-#use warnings; #it's ok if we can't load warnings, but the author should
 use strict;
 use Proc::Exists::Configuration;
 use vars qw (@ISA @EXPORT_OK $VERSION); 
+eval { require warnings; }; #it's ok if we can't load warnings
 
 require Exporter;
 use base 'Exporter';
 @EXPORT_OK = qw(pexists);
 
-$VERSION = '0.92_01';
+$VERSION = '0.99_01';
 
-eval {
-	require XSLoader;
-	XSLoader::load('Proc::Exists', $VERSION); 
-}; if($@) {
+my $use_pureperl = $Proc::Exists::Configuration::want_pureperl; 
+if(!$use_pureperl) {
+	eval {
+		require XSLoader;
+		XSLoader::load('Proc::Exists', $VERSION); 
+	}; if($@) {
+		#NOTE: don't need to worry about i18n, {XS|Dyna}Loader complain in english.
+		if($@ =~ /Proc::Exists\s+object\s+version\s+\S+\s+does\s+not\s+match\s+bootstrap\s+parameter/ ) {
+			warn "WARNING: it looks like you have a previous Proc::Exists ".
+			     "version's object file(s) somewhere in \@INC! you will have ".
+			     "to remove these and reinstall Proc::Exists. for now, we are ".
+			     "falling back to pureperl, expect degraded performance: $@\n"; 
+		} else {
+			warn "WARNING: can't load XS. falling back to pureperl, ".
+			     "expect degraded performance: $@\n"; 
+		}
+		$use_pureperl = 1;
+	}
+}
+
+if($use_pureperl) {
 	#warn "using pure perl mode, expect degraded performance\n";
-	my $EPERM = $Proc::Exists::Configuration::EPERM; 
-	my $ESRCH = $Proc::Exists::Configuration::ESRCH; 
 	my $pp_pexists = sub {
 		my @pids = @_; 
-		my %args = %{pop(@pids)} if (ref($pids[-1]));
+		my %args = ref($pids[-1]) ? %{pop(@pids)} : ();
 
-		if(wantarray && %args) {
-			die "can't specify all argument in list context" if($args{all}); 
-			die "can't specify any argument in list context" if($args{any}); 
+		die "can't specify both 'any' and 'all' arg" if($args{all} && $args{any}); 
+		if(wantarray) {
+			die "can't specify 'all' argument in list context" if($args{all}); 
+			die "can't specify 'any' argument in list context" if($args{any}); 
 		}
 
 		my @results; 
@@ -44,9 +60,9 @@ eval {
 			if (kill 0, $pid) {
 				$ret = 1;
 			} else {
-				if($! == $EPERM) {
+				if($! == $Proc::Exists::Configuration::EPERM) {
 					$ret = 1;
-				} elsif($! == $ESRCH) {
+				} elsif($! == $Proc::Exists::Configuration::ESRCH) {
 					$ret = 0;
 				} elsif($^O eq "MSWin32") {
 					die "can't do pure perl on MSWin32 - \$!: (".(0+$!)."): $!"; 
@@ -54,7 +70,7 @@ eval {
 					die "unknown numeric \$!: (".(0+$!)."): $!, pureperl, OS: $^O"; 
 				}
 			}
-	
+
 			if($ret) {
 				return $pid if($args{any}); 
 				push @results, $pid; 
@@ -62,6 +78,9 @@ eval {
 				return 0;
 			}
 		}
+		#NOTE: as documented in the pod, any returns undef for false,
+		#      because some systems use pid==0         
+		return if($args{any}); 
 		return wantarray ? @results : scalar @results; 
 	};
 	*pexists = \&$pp_pexists; 
@@ -71,15 +90,14 @@ eval {
 
 	my $xs_pexists = sub {
 		my @pids = @_; 
-		my %args = %{pop(@pids)} if (ref($pids[-1]));
+		my %args = ref($pids[-1]) ? %{pop(@pids)} : ();
 
 		if(wantarray) {
-			if(%args) {
-				die "can't specify all argument in list context" if($args{all}); 
-				die "can't specify any argument in list context" if($args{any}); 
-			}
+			die "can't specify 'all' argument in list context" if($args{all}); 
+			die "can't specify 'any' argument in list context" if($args{any}); 
 			return _list_pexists([@pids]); 
 		} else {
+			die "can't specify both 'any' and 'all' arg" if($args{all} && $args{any}); 
 			return _scalar_pexists([@pids], $args{any} || 0, $args{all} || 0);
 		}
 	};
@@ -89,11 +107,8 @@ eval {
 }
 
 # !wantarray        : return number of matches
-# !wantarray && any : return pid of first match if any match, else 0
-#   FIXME: TODO: testcases
-#   FIXME: TODO: what about systems where 0 is a valid pid?
-#   perhaps return undef instead of 0 in this case?
-# !wantarray && all : return 1 if all match, else undef
+# !wantarray && any : return pid of first match if any match, else undef
+# !wantarray && all : return a true value if all match, else a false value
 #  wantarray        : return list of matching pids
 #  wantarray && any : undefined, makes no sense
 #   ALTERNATELY: could return list of size one with first matching pid, 
@@ -106,12 +121,7 @@ __END__
 
 =head1 NAME
 
-Proc::Exists - quickly check for process existence
-
-
-=head1 VERSION
-
-This document describes Proc::Exists
+Proc::Exists - quickly and portably check for process existence
 
 
 =head1 SYNOPSIS
@@ -121,8 +131,9 @@ This document describes Proc::Exists
    my $dead_or_alive        = pexists($pid); 
    my @survivors            = pexists(@pid_list); 
    my $nsurvivors           = pexists(@pid_list); 
-   my $pid_of_one_survivor  = pexists(@pid_list, {any => 1});
-   my $bool_all_pids_lived  = pexists(@pid_list, {all => 1});
+   my $all_pids_survived    = pexists(@pid_list, {all => 1});
+   my $pid_of_one_survivor_or_undef_if_all_are_dead =
+                              pexists(@pid_list, {any => 1});
 
   
 =head1 FUNCTIONS
@@ -131,10 +142,11 @@ This document describes Proc::Exists
 
 Supported arguments are 'any' and 'all', as shown above.
 
-The 'any' argument returns the pid of the first process found.
-
 In list context, giving the 'any' or 'all' arguments will error out.
 
+The 'any' argument returns the pid of the first process found, or undef 
+if none are found. Note that on some systems, 0 is a valid and usually 
+extant pid - see B<CAVEATS> for more information.
 
 =head1 DESCRIPTION
 
@@ -151,29 +163,24 @@ the pid of the process to check and examining the result and errno.
  * any os with a POSIX layer or win32
  * Test::More if you want to run 'make test'
 
-
-=head1 INCOMPATIBILITIES
-
 It's possible that if you don't have a C compiler, and you're
 running an "obscure" UNIX-y OS (read: not linux, *BSD, solaris,
 or Mac OS X), you might not pass make test. This is because we need
 to compare the value of $! after a call to kill() with EPERM
-and ESRCH. Not wanting to rely on POSIX, we determine EPERM 
-and ESRCH at build (Makefile.PL) time, by using POSIX if it exists
--- but using the common values of EPERM==1 and ESRCH==3 if we can't 
-load POSIX. If you find yourself on such a system, your best bet is to 
-look up EPERM and ESRCH (try grepping for them down /usr/include or 
-wherever your headers are kept). If you get hits back, you can edit 
-Exists/Configuration.pm and add your values there, and re-run the build 
-process. Whether you were successful or not, please send a description
-of what you tried, as well as the output of perl -V and the results of 
-perl misc/gather-info.pl to B<< <ski-cpan@allafrica.com> >> - making
-sure to include Proc::Exists in the subject line (or else I won't read 
-it!) If you had no success, hopefully I'll be able to provide a patch 
-for you, and a fix/workaround for the next release of Proc::Exists. By 
-the way, don't be afraid that we send signal 0 to your processes - kill 
-with signal 0 only indicates if a signal may be sent, it does not 
-actually send any signal, and so will not disrupt any process.
+and ESRCH. Not wanting to rely on Errno or POSIX, we determine EPERM 
+and ESRCH at build (Makefile.PL) time, by using POSIX or Errno if
+it exists -- but using the common values of EPERM==1 and ESRCH==3 if
+we can't load POSIX. If you find yourself on such a system, your best 
+bet is to look up EPERM and ESRCH (try grepping for them down 
+/usr/include or wherever your headers are kept). If you get hits
+back, you can edit Exists/Configuration.pm, add your values there, 
+and re-run the build process. Whether you were successful or not, 
+please send a descriptionof what you tried, as well as the output of 
+perl -V and the results of perl misc/gather-info.pl to B<< 
+<ski-cpan@allafrica.com> >> - making sure to include Proc::Exists in the 
+subject line (or else I won't read it!) If you had no success, 
+hopefully I'll be able to provide a patch for you, and a fix/workaround 
+for the next release of Proc::Exists.
 
 There is no pure perl implementation under Windows. The solution
 is to use Strawberry Perl L<http://strawberryperl.com/>.
@@ -182,6 +189,30 @@ Any other OS without a POSIX emulation layer will probably be
 completely non-functional (unless it implements C<kill()>).
 
 
+=head1 CAVEATS
+
+The 'any' argument returns the pid of the first process found, or undef 
+if none are found. Note that on some systems (e.g. OSX), 0 is a valid 
+pid (that almost always exists). Since the 'any' mode will return the 
+first pid that matches, a return value of 0 can indicate that a pid was 
+found. To avoid this problem, make sure you check whether 'any' mode 
+found anything by checking for defined-ness, not whether the result 
+evaluates to true or false in boolean context, like so:
+
+	if(defined(pexists(@pid_list, {any => 1})));
+
+B<< DO NOT >> use this idiom:
+
+	if(pexists(@pid_list, {any => 1}));
+
+Note also that this caveat does NOT apply for "plain" pexists() (ie 
+without 'any' or 'all' arguments), because in scalar context a count is 
+returned, so pexists(0) returns 1 when pid 0 exists, as expected. we 
+only get a pid with 'any' or when we are in list context, and in the 
+latter case, an array of length 1 containing a false value evaluates 
+true in boolean context.
+
+
 =head1 BUGS AND LIMITATIONS
 
 Please report any bugs or feature requests through the
@@ -195,7 +226,7 @@ Proc::Exists in the subject line if you want me to read your message.
 
 =head1 LICENCE AND COPYRIGHT
 
-Copyright (c) 2008, Brian Szymanski B<< <ski-cpan@allafrica.com> >>.
+Copyright (c) 2008-2009, Brian Szymanski B<< <ski-cpan@allafrica.com> >>.
 All rights reserved.
 
 This module is free software; you can redistribute it and/or
